from typing_extensions import AsyncIterable, Sequence
from dataclasses import dataclass
from abc import ABC, abstractmethod
from decimal import Decimal
from datetime import datetime
from trading_sdk.types import Network

@dataclass
class Deposit:
  @dataclass
  class Fee:
    asset: str
    amount: Decimal
  id: str
  amount: Decimal
  asset: str
  time: datetime
  address: str | None = None
  network: Network | None = None
  fee: Fee | None = None
  memo: str | None = None

class DepositHistory(ABC):
  @abstractmethod
  def deposit_history(
    self, *, asset: str | None = None,
    start: datetime,
    end: datetime,
  ) -> AsyncIterable[Sequence[Deposit]]:
    """Fetch your deposits.
    
    - `asset`: if given, retrieves deposits for this asset.
    - `start`: if given, retrieves deposits after this time.
    - `end`: if given, retrieves deposits before this time.
    """
    ...